@@ -19,7 +19,7 @@
  *
  * Inspired by nsyslog, originally written by Darren Reed.
  *
- * $Id: affile.c,v 1.36 2001/04/14 08:02:17 bazsi Exp $
+ * $Id: affile.c,v 1.38 2001/07/11 14:10:16 bazsi Exp $
  *
  ***************************************************************************/
 
@@ -77,16 +77,20 @@ static int do_open_file(struct ol_string *name, int flags,
 			else if (errno == ENOENT) {
 				if (mkdir(name->data, dir_mode) == -1)
 					return 0;
-				chown(name->data, dir_uid, dir_gid);
-				chmod(name->data, dir_mode);
+				if (dir_uid != -1 || dir_gid != -1)
+					chown(name->data, dir_uid, dir_gid);
+				if (dir_mode != -1)
+					chmod(name->data, dir_mode);
 			}
 			*p = '/';
 			p = strchr(p + 1, '/');
 		}
 		*fd = open(name->data, flags, mode);
 	}
-	chown(name->data, uid, gid);
-	chmod(name->data, mode);
+	if (uid != -1 || gid != -1)
+		chown(name->data, uid, gid);
+	if (mode != -1)
+		chmod(name->data, mode);
 	return *fd != -1;
 }
 
@@ -110,11 +114,11 @@ static int do_init_affile_source(struct log_handler *c, struct syslog_config *cf
   		flags = O_RDWR | O_NOCTTY | O_NONBLOCK;
   	else
 	  	flags = O_RDONLY | O_NOCTTY | O_NONBLOCK;
-	if (do_open_file(self->name, flags, 0, 0, 0, 0, 0, 0, 0, &fd)) {
+	if (do_open_file(self->name, flags, -1, -1, 0, 0, 0, 0, 0, &fd)) {
 		struct resource_node *res;
 
 		lseek(fd, 0, SEEK_END);
-		self->src = io_read(make_io_fd(cfg->backend, fd), 
+		self->src = io_read(make_io_fd(cfg->backend, fd, self->name), 
 			make_log_reader(0, c), 
 			NULL);
 		res = REMEMBER_RESOURCE(cfg->resources, &self->src->super.super);
@@ -214,7 +218,7 @@ do_init_dest_writer(struct log_handler *c, struct syslog_config *cfg, struct per
 	                 self->owner->flags & AFFILE_CREATE_DIRS, &fd)) {
 
 		self->dest = 
-			io_write(make_io_fd(cfg->backend, fd), 
+			io_write(make_io_fd(cfg->backend, fd, self->expanded_fname), 
 				 make_pkt_buffer_ext(self->owner->sync_freq, 
 				                     MAX(self->owner->super.log_fifo_size + 1, 
 					                 self->owner->sync_freq)),
@@ -412,12 +416,13 @@ struct macro_def {
 #define M_HOUR     9
 #define M_MIN      10
 #define M_SEC      11
+#define M_WEEKDAY  12
 
-#define M_FULLHOST 12
-#define M_HOST     13
-#define M_PROGRAM  14
+#define M_FULLHOST 13
+#define M_HOST     14
+#define M_PROGRAM  15
 
-#define M_MESSAGE  15
+#define M_MESSAGE  16
 
 int append_string(char **dest, int *left, char *str, int length, int escape)
 {
@@ -513,30 +518,9 @@ void expand_macro(struct syslog_config *cfg, int id, int escape, char **dest, in
 		}
 		break;
 	}
-	case M_DATE: {
-		/* date */
-		struct tm *tm;
-
-		tm = localtime(&msg->stamp);
-                length = strftime(*dest, *left - 1, "%h %e %H:%M:%S", tm);
-		break;
-	}
-	case M_FULLDATE: {
-		/* date */
-		struct tm *tm;
-
-		tm = localtime(&msg->stamp);
-                length = strftime(*dest, *left - 1, "%Y %h %e %H:%M:%S", tm);
-		break;
-	}
-	case M_ISODATE: {
-		/* date */
-		struct tm *tm;
-
-		tm = localtime(&msg->stamp);
-                length = strftime(*dest, *left - 1, "%Y-%m-%dT%H:%M:%S%z", tm);
-		break;
-	}
+	case M_FULLDATE:
+	case M_ISODATE:
+	case M_WEEKDAY:
 	case M_YEAR:
 	case M_MONTH:
 	case M_DAY: 
@@ -552,6 +536,9 @@ void expand_macro(struct syslog_config *cfg, int id, int escape, char **dest, in
 			tm = localtime(&msg->stamp);
 
 		switch (id) {
+		case M_WEEKDAY:
+	                length = strftime(*dest, *left - 1, "%a", tm);			
+			break;
 		case M_YEAR:
 			length = snprintf(*dest, *left, "%04d", tm->tm_year + 1900);
 			break;
@@ -570,6 +557,15 @@ void expand_macro(struct syslog_config *cfg, int id, int escape, char **dest, in
 		case M_SEC:
 			length = snprintf(*dest, *left, "%02d", tm->tm_sec);
 			break;
+		case M_ISODATE:
+	                length = strftime(*dest, *left - 1, "%Y-%m-%dT%H:%M:%S%z", tm);
+	                break;
+	        case M_FULLDATE:
+	                length = strftime(*dest, *left - 1, "%Y %h %e %H:%M:%S", tm);
+	        	break;
+	        case M_DATE:
+	                length = strftime(*dest, *left - 1, "%h %e %H:%M:%S", tm);
+	                break;
 		}
 		break;
 	case M_MESSAGE: {
@@ -603,6 +599,7 @@ expand_macros(struct syslog_config *cfg, struct ol_string *template, int templat
 		{ "HOUR", M_HOUR },
 		{ "MIN", M_MIN },
 		{ "SEC", M_SEC },
+		{ "WEEKDAY", M_WEEKDAY },
 		
 		{ "HOST", M_HOST },
 		{ "FULLHOST", M_FULLHOST },
