@@ -19,7 +19,7 @@
  *
  * Inspired by nsyslog, originally written by Darren Reed.
  *
- * $Id: afunix.c,v 1.17 2001/05/04 12:37:43 bazsi Exp $
+ * $Id: afunix.c,v 1.18 2001/07/08 17:04:21 bazsi Exp $
  *
  ***************************************************************************/
 
@@ -240,7 +240,7 @@ int do_init_afunix_source(struct log_handler *c,
 			
 	case AFSOCKET_DGRAM: {
 		struct afsocket_source_connection *conn;
-		struct io_fd *client = make_io_fd(cfg->backend, fd);
+		struct io_fd *client = make_io_fd(cfg->backend, fd, NULL);
 		int res;
 				
 		self->super.num_connections = 1;
