@@ -19,7 +19,7 @@
  *
  * Inspired by nsyslog, originally written by Darren Reed.
  *
- * $Id: center.c,v 1.17 2001/05/04 12:37:43 bazsi Exp $
+ * $Id: center.c,v 1.18 2001/07/08 17:04:21 bazsi Exp $
  *
  ***************************************************************************/
 
@@ -50,11 +50,49 @@ struct log_endpoint_info *make_log_endpoint_info(int type, const char *name)
 
 /* log_connection */
 
-struct log_connection *make_log_connection(struct log_endpoint_info *nodes)
+struct log_connection *make_log_connection(struct log_endpoint_info *nodes,
+	UINT32 flags)
 {
 	NEW(log_connection, self);
+	struct log_endpoint_info *last_src, *last_filter, *last_dest;
 	
-	self->nodes = nodes;
+	self->flags = flags;
+
+	/* Preseve the order of endpoints, it might be important */
+	for (last_src = last_filter = last_dest = NULL; nodes; nodes = nodes->next) {
+		switch (nodes->type)
+		{
+		case EP_SOURCE:
+			if (last_src) {
+				last_src->next = nodes;
+				last_src = last_src->next;
+			} else
+				self->sources = last_src = nodes;
+			break;
+		case EP_FILTER:
+			if (last_filter) {
+				last_filter->next = nodes;
+				last_filter = last_filter->next;
+			} else
+				self->filters = last_filter = nodes;
+			break;
+		case EP_DESTINATION:
+			if (last_dest) {
+				last_dest->next = nodes;
+				last_dest = last_dest->next;
+			} else
+				self->destinations = last_dest = nodes;
+			break;
+		} /* switch in for */
+	}
+
+	if (last_src)
+		last_src->next = NULL;
+	if (last_filter)
+		last_filter->next = NULL;
+	if (last_dest)
+		last_dest->next = NULL;
+
 	return self;
 }
 
@@ -67,53 +105,62 @@ struct log_connection *make_log_connection(struct log_endpoint_info *nodes)
          (connections object log_connection)))
 */
 
-/* FIXME: make the below algorithm a bit quicker */
 static void do_distribute_log(struct log_handler *c, struct log_info *msg)
 {
 	CAST(log_center, self, c);
 	struct log_connection *p;
-	int match = 0;
-	
-	for (p = self->connections; p; p = p->next) {
-		struct log_endpoint_info *s, *d;
+	int match;
 
-		if (p->flags & LOG_CONN_DEFAULT) {
-			if (match)
-				continue;
-			for (d = p->nodes; d; d = d->next) {
-				if (d->type == EP_DESTINATION) {
-					assert(d->ref);
-					HANDLE_LOG(d->ref, log_info_use(msg));
-				}
-			}
+	for (match = 0, p = self->connections; p; p = p->next) {
+		struct log_endpoint_info *s, *f, *d;
+
+		if (match && (p->flags & LOG_CONN_FALLBACK))
 			continue;
-		}
-		for (s = p->nodes; s; s = s->next) {
-			if ((s->type == EP_SOURCE) && ((struct ol_object *) msg->source == s->ref)) {
-				/* 
-				 * so we found this source in a log statement,
-				 * distribute message
-				 */
-				struct log_endpoint_info *f;
 
-				for (f = p->nodes; f; f = f->next) {
-					if (f->type == EP_FILTER && 
-					    !FILTER_RULE_EVAL(((struct log_filter *) f->ref)->root, (struct log_filter *) f->ref, msg))
-						goto next_connection;
-
-				}
-				for (d = p->nodes; d; d = d->next) {
-					match = 1;
-					if (d->type == EP_DESTINATION) {
-						assert(d->ref);
-						HANDLE_LOG(d->ref, log_info_use(msg));
-					}
-				}
+		/* Source checking (source is a kind of filter) */
+		if (!(p->flags & LOG_CONN_CATCHALL)) {
+			for (s = p->sources; s; s = s->next) {
+				assert(s->ref);
+				if ((struct ol_object *) msg->source == s->ref)
+					break;
 			}
+			if (!s)
+				goto next_connection;
 		}
+
+		/* Filter checking */
+		for (f = p->filters; f; f = f->next) {
+			assert(f->ref);
+			if (!FILTER_RULE_EVAL(((struct log_filter *) f->ref)->root,
+					(struct log_filter *) f->ref, msg))
+				goto next_connection;
+		}
+
+		/* Matched, let's distribute it */
+		match = 1;
+		for (d = p->destinations; d; d = d->next) {
+			assert(d->ref);
+			HANDLE_LOG(d->ref, log_info_use(msg));
+		}
+
+		if (p->flags & LOG_CONN_FINAL)
+			break;
 		
 	next_connection:
 	}
+
+	if (!match) {
+		for (p = self->connections; p; p = p->next) {
+			if (p->flags & LOG_CONN_FALLBACK) {
+				struct log_endpoint_info *d;
+				
+				for (d = p->destinations; d; d = d->next) {
+					assert(d->ref);
+					HANDLE_LOG(d->ref, log_info_use(msg));
+				}				
+			}
+		}
+	}
 	log_info_free(msg);
 }
 
