@@ -19,7 +19,7 @@
  *
  * Inspired by nsyslog, originally written by Darren Reed.
  *
- * $Id: cfgfile.c,v 1.33 2001/02/24 12:38:03 bazsi Exp $
+ * $Id: cfgfile.c,v 1.35 2001/07/08 17:04:21 bazsi Exp $
  *
  ***************************************************************************/
 
@@ -184,6 +184,8 @@ make_persistent_config(void)
 
 /* cfgfile */
 #define RESOLVE_VAR(s, first, next, i) \
+do \
+{ \
         for (s = first; s; s = s->next) { \
 	         if (i->name->length == s->name->length && \
 		     strncmp(i->name->data, s->name->data, i->name->length) == 0) { \
@@ -195,7 +197,7 @@ make_persistent_config(void)
 		 werror("unresolved reference: %S\n", i->name); \
 		 return 0; \
 	} \
-
+} while (0)
 
 static int do_init_config(struct syslog_config *self, struct persistent_config *p)
 {
@@ -205,7 +207,7 @@ static int do_init_config(struct syslog_config *self, struct persistent_config *
 	struct log_connection *c;
 	struct log_endpoint_info *i;
 	struct log_handler *center;
-	int res;
+	int res, n;
 
 	gc_idle_threshold = self->gc_idle_threshold;
 	gc_busy_threshold = self->gc_busy_threshold;
@@ -225,32 +227,21 @@ static int do_init_config(struct syslog_config *self, struct persistent_config *
 	}
 
 	if (!self->internal) {
-		werror("Warning: Internal messages are not referenced, they'll go to /dev/null\n");
+		werror("Warning: No source refers to internal messages, they'll go to /dev/null\n");
 	}
 	
-	for (c = self->connections; c; c = c->next) {
-		for (i = c->nodes; i; i = i->next) {
-			switch (i->type) {
-			case EP_SOURCE:
-				RESOLVE_VAR(s, self->sources, next_source_group, i);
-				break;
-			case EP_FILTER:
-				if (i->name->length == 7 &&
-				    memcmp(i->name->data, "DEFAULT", 7) == 0)
-					c->flags |= LOG_CONN_DEFAULT;
-				else
-					if ((c->flags & LOG_CONN_DEFAULT) == 0) {
-						RESOLVE_VAR(f, self->filters, next_filter, i);
-					}
-					else {
-						werror("Default statement may not have additional filters.\n");
-						return 0;
-					}
-				break;
-			case EP_DESTINATION:
-				RESOLVE_VAR(d, self->destinations, next_dest_group, i);
-				break;
-			}
+	for (c = self->connections, n = 0; c; c = c->next, n++) {
+		if (!c->sources && !(c->flags & LOG_CONN_CATCHALL)) {
+			werror("Warning: connection %i doesn't have any sources\n", n);
+		}
+		for (i = c->sources; i; i = i->next) {
+			RESOLVE_VAR(s, self->sources, next_source_group, i);
+		}
+		for (i = c->filters; i; i = i->next) {
+			RESOLVE_VAR(f, self->filters, next_filter, i);
+		}
+		for (i = c->destinations; i; i = i->next) {
+			RESOLVE_VAR(d, self->destinations, next_dest_group, i);
 		}
 	}
 
