@@ -19,7 +19,7 @@
  *
  * Inspired by nsyslog, originally written by Darren Reed.
  *
- * $Id: log.c,v 1.22 2001/05/04 12:37:43 bazsi Exp $
+ * $Id: log.c,v 1.23 2001/06/11 07:44:11 bazsi Exp $
  *
  ***************************************************************************/
 
@@ -106,7 +106,7 @@ static void parse_log_msg(struct log_info *lm, UINT32 length, UINT8 *data)
 			strptime(lm->date->data, "%b %e %H:%M:%S", &tm);
 			tm.tm_isdst = -1;
 			tm.tm_year = nowtm->tm_year;
-			if (tm.tm_mon > nowtm->tm_year)
+			if (tm.tm_mon > nowtm->tm_mon)
 				tm.tm_year--;
 			lm->stamp = mktime(&tm);
 		}
